/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
 - NO WARRANTIES
     This software is provided 'As Is', without any express or implied
     warranty of any kind, including but not limited to any warranties 
     of merchantability, noninfringement, or fitness of a particular 
     purpose. The Copyright holders do not warrant or assume responsibility
     for the accuracy or completeness of any information contained within
     this software. 
 - LIMITATION OF LIABILITY
     In no event shall the copyright holders be liable for any damages 
     (including, without limitation, lost profits, business interruption, 
     or lost information) rising out of use of or inability to use this 
     software, even if advised of the possibility of such damages.  In no
     event will the copyright holders be liable for loss of data or for 
     indirect, special, incidental, consequential (including lost profit),
     or other damages based in contract, tort or otherwise. The copyright
     holders shall have no liability with respect to the content of the 
     software or any part thereof, including but not limited to errors or 
     omissions contained therein, libel, infringements of rights of publicity,
     privacy, trademark rights, business interruption, personal injury, 
     loss of privacy, moral rights or the disclosure of confidential
     information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
  source files and, if distributed in compiled form, must be duplicated in 
  documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
  software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
  product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

#include "../plugin.h"

// Hybrid plugin example.
// Based on the ASYNC pre-parser.  Adds a new signal to monitor a direction signal.
// Prints fields in GREEN when the direction is HIGH, and in BLUE when it is low.
// Also starts a new frame every time the direction switches.

// ASYNC specific event flags
#define BRKEVENT   0x80
#define ENDEVENT   0x40
#define PERROR     0x20
#define FERROR     0x10
#define ADDRBYTE   0x02
#define DATAEVENT  0x01

static bool ShowFieldIdle;
static bool dir;
static bool olddir;
static bool newframe;
static bool dirinvert;
static Data64 dirmask;

void OnLoad() {}       // no one-time initialization needed
void OnUnLoad() {}     // no final cleanup needed
void SetInitItem(unsigned char ID, unsigned char subID, int value) {} // don't care about globals
void EndOfData() {}    // no end-of-parsing finalization needed


enum field_formats {FFtxdata,FFrxdata,FFperr,FFferr,FFbrk,FFEND};

void GetStrList(int ID, vector<string> &strl)
{	
	switch (ID) 
	{  // 5 (lookuptables) returns empty list
	case 0: strl.push_back("Half Duplex example V1.0"); break;
	case 1: 
		strl.push_back("Show Field Idle,checkbox,false");
		strl.push_back("Direction,chanselect,1,1,1,true,false");
		break;
	case 2: // field formats 
		strl.assign(FFEND,"");
		strl[FFtxdata]  = "DATA,GREEN,BLACK,{7:0}"; 
		strl[FFrxdata] = "DATA,BLUE,WHITE,{7:0}";
		strl[FFperr] = "ParityError,RED,WHITE,P";
		strl[FFferr] = "FrameError,RED,WHITE,F";
		strl[FFbrk]  = "BREAK,YELLOW,BLACK,BREAK";
		break; 
	case 3:  strl.push_back("ASYNC"); break;      // use ASYNC pre-parser
	case 4:  strl.push_back("1");	break;        // require framework version 1 or above
	}
}


void SetCfgItem(unsigned char ID, unsigned char subID, int value) 
{ 
	switch (ID)
	{
	case 0:
		if (subID == 0) ShowFieldIdle = (value != 0);
		break;
	case 1:
		switch (subID)
		{
		case 0: dirinvert = (value & 0x01); break;
		case 1: dirmask.lowint = value;   break;
		case 2: dirmask.highint = value;  break;
		}
		break;
	}
}


void StartOfData() 
{    
	newframe  = true;
}


void Parse(int64 timestamp, Data64 data)
{
	// handle raw data events first
	if (data.bytes[7] == 0x90)
	{
		dir = ((data.i64 & dirmask.i64) != 0);
		if (dirinvert) dir = !dir;
		if (dir != olddir) newframe = true;
		olddir = dir;
	}

	// now handle pre-parser data events
	if (data.bytes[7] == 0x80)
	{
		if (data.bytes[6] & DATAEVENT)
		{   // output it as start-of-FRAME or start-of-FIELD, depending on change in direction
			// also select color based on direction level
		    if (newframe) StartFrame(timestamp,data,dir?FFtxdata:FFrxdata);  
			else StartField(timestamp,data,dir?FFtxdata:FFrxdata);  
			newframe = false;
		}

		// note use of 'else'. Multiple event flags can be set along with the data event to give us
		// early notice of any errors. However, we will receive additional events later (with individual
		// timestamps) to handle the errors. We choose to handle them individually.
		else if (data.bytes[6] & PERROR)  // handle parity error events
			StartField(timestamp,data,FFperr);  // output it with Parity error field format

		else if (data.bytes[6] & FERROR)  // handle framing error events
			StartField(timestamp,data,FFferr);  // output it with Frame error field format

		else if (data.bytes[6] & BRKEVENT)  // handle BREAK events
			StartField(timestamp,data,FFbrk);  // output it with BREAK field format 

		else if (ShowFieldIdle && (data.bytes[6] & ENDEVENT))  // handle end-of-char events
			EndField(timestamp);   // start IDLE (doesn't use formats)

	}
}