/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
 - NO WARRANTIES
     This software is provided 'As Is', without any express or implied
     warranty of any kind, including but not limited to any warranties 
     of merchantability, noninfringement, or fitness of a particular 
     purpose. The Copyright holders do not warrant or assume responsibility
     for the accuracy or completeness of any information contained within
     this software. 
 - LIMITATION OF LIABILITY
     In no event shall the copyright holders be liable for any damages 
     (including, without limitation, lost profits, business interruption, 
     or lost information) rising out of use of or inability to use this 
     software, even if advised of the possibility of such damages.  In no
     event will the copyright holders be liable for loss of data or for 
     indirect, special, incidental, consequential (including lost profit),
     or other damages based in contract, tort or otherwise. The copyright
     holders shall have no liability with respect to the content of the 
     software or any part thereof, including but not limited to errors or 
     omissions contained therein, libel, infringements of rights of publicity,
     privacy, trademark rights, business interruption, personal injury, 
     loss of privacy, moral rights or the disclosure of confidential
     information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
  source files and, if distributed in compiled form, must be duplicated in 
  documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
  software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
  product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

// Track2-full   - decodes track 2 from magnetic stipe cards.
// Full parser.  Demonstrates channel extraction, edge detection, using chanselect INVERT options. 
//
// NOTE: this is a SYNC type signal and could have been written as a mini-parser.  However, it is cleaner
// to write this one as a full parser because most of the SYNC pre-parser options are not used.  By writing a full
// parser, we were able to present only the pertinent options to the user.

#include "../plugin.h"

static Data64 clkmask;
static Data64 datmask;
static bool clkinvert;
static bool datinvert;
static bool parity;
static int shifter;
static int bitcount;
static bool firsttime;
static bool showparityfield;
static bool showrawdata;

//  configuration items (case 1)
static enum cfgnames { CLK_MASK,DATA_MASK,RAWDATA,CFGEND};

// field formats (case 2)
static enum fieldnames { F_DATA, F_DATAERROR, F_END };

// translation table - translates MSB first 0x00-0x0F to LSB first 0x30-0x3F
const int xlat [16] = {0x30,0x38,0x34,0x3C,0x32,0x3A,0x36,0x3E,0x31,0x39,0x35,0x3D,0x33,0x3B,0x37,0x3F};

void OnLoad() {}       // no one-time initialization needed
void OnUnLoad() {}     // no final cleanup needed
void SetInitItem(unsigned char ID, unsigned char subID, int value) {} // don't care about globals
void EndOfData() {}    // no end-of-parsing finalization needed

// get our configuration
void GetStrList(int ID, vector<string> &strl)
{	
	switch (ID) 
	{  
	case 0:
		strl.push_back("Track2-full Example V1.0");
		break;

	case 1:
		strl.assign(CFGEND,"");
		strl[CLK_MASK]  = "Clock:,chanselect,1,1,1,true,false";
		strl[DATA_MASK] = "Data:,chanselect,1,1,1,true,false";
		strl[RAWDATA]   = "Show Raw Data (0-F),checkbox,false";
		break;

	case 2: 
		strl.assign(F_END,"");
		strl[F_DATA]      = "Data,,,{7:0}"; 
		strl[F_DATAERROR] = "Data Error,RED,WHITE,{7:0}";
		break; 
	case 3:  strl.push_back("RAW"); break;      // use RAW pre-parser
	case 4:  strl.push_back("1");	break;      // require framework version 1 or above
		//case  5 (lookuptables) returns empty list
	}
}


void SetCfgItem(unsigned char ID, unsigned char subID, int value)  
{
	switch (ID)
	{
	case DATA_MASK:  // data bus mask
		switch (subID)
		{
		case 0: datinvert = (value & 0x01); break; 
		case 1: datmask.lowint = value;   break;
		case 2: datmask.highint = value;  break;
		}
		break;

	case CLK_MASK:   // clock bit mask
		switch (subID)
		{
		case 0: clkinvert = (value & 0x01); break; 
		case 1: clkmask.lowint = value;  break;
		case 2: clkmask.highint = value; break;
		}
		break;
	case RAWDATA:
		if (subID == 0) showrawdata = (value != 0);
		break;
	}
}


void StartOfData() 
{
	firsttime = true;
}

void Parse(int64 timestamp, Data64 data)
{ 
	static bool oldclk;
	static int64 starttime;
	bool clk;
	bool dat;

	clk = ((data.i64 & clkmask.i64) != 0);
	if (clkinvert) clk = !clk;


	if (firsttime)  // first sample...initialize things
	{
		shifter = 0;
		bitcount = 0;
		parity = false;    // odd parity
		oldclk = clk;      // note starting level of clock bit
		firsttime = false;
	}

	// see if this is a clock edge we want
	else if (!oldclk && clk)
	{
		dat = ((data.i64 & datmask.i64) != 0);      // extract the data channel
		if (datinvert) dat = !dat;     // invert data if told to

		if (bitcount < 4)  // NOT parity bit, shift in the current data bit
		{
			if (bitcount == 0) starttime = timestamp; // note start of symbol time
			bitcount += 1;
			shifter <<= 1;
			if (dat) 
			{ 
				shifter += 1;
				parity = !parity; 
			}
		}

		else   // this is the final (parity) bit
		{
			// send symbol
			// NOTE: we gathered the data bits MSB first, but the spec calls for LSB first.
			// Also, the spec calls for a translation of 0-F = 0x30-0x3F
			// An efficient way of handling both issues at once is a translation table
			data.i64 = xlat[shifter];

            if (showrawdata)
				data.i64 -= 0x30;   // remove character '0' offset

			// check parity bit 
			if (parity == dat)   // parity error
				StartField(starttime,data,F_DATAERROR);  // output the symbol with ERROR DATA field format
			else  // parity OK...
				StartField(starttime,data,F_DATA);  // output the symbol with normal DATA field format

			// terminate field at the parity bit time
			EndField(timestamp);  

			// reset, ready for the next symbol
			bitcount = 0;
			shifter = 0;
			parity = false;
		}
	}
    oldclk = clk;    // update last known clock level
}